import "dart:collection";
import "vector.dart";

class Array2D<T> extends IterableBase<T> {
  final List<T> _data;
  final int rows;
  final int cols;

  Array2D(this.rows, this.cols, [T defaultValue])
      : _data = new List<T>(rows * cols) {
    if (defaultValue != null)
      for (int i = 0; i < _data.length; i++) _data[i] = defaultValue;
  }

  int _cartesianToIndex(int x, int y) => y * cols + x;

  void set(int x, int y, T element) {
    _data[_cartesianToIndex(x, y)] = element;
  }

  T get(int x, int y) => _data[_cartesianToIndex(x, y)];

  T operator [](Vector i) => _data[_cartesianToIndex(i.x, i.y)];

  void operator []=(Vector i, T element) {
    _data[_cartesianToIndex(i.x, i.y)] = element;
  }

  Iterator<T> get iterator => _data.iterator;
}
