import "dart:math" as math;

class Vector {
  final int x;
  final int y;

  const Vector(this.x, this.y);

  Vector.fromVector(Vector v)
      : x = v.x,
        y = v.y;

  int get area => x * y;

  int get lengthSquared => x * x + y * y;

  num get length => math.sqrt(this.lengthSquared);

  bool operator ==(Object o) {
    if (o is Vector) return x == o.x && y == o.y;

    return false;
  }

  bool operator >(Object o) {
    if (o is Vector)
      return lengthSquared > o.lengthSquared;
    else if (o is num) return lengthSquared > o * o;

    throw ArgumentError("Operand's type must be int or Vector");
  }

  bool operator >=(Object o) {
    if (o is Vector)
      return lengthSquared >= o.lengthSquared;
    else if (o is num) return lengthSquared >= o * o;

    throw ArgumentError("Operand's type must be int or Vector");
  }

  bool operator <(Object o) {
    if (o is Vector)
      return lengthSquared < o.lengthSquared;
    else if (o is num) return lengthSquared < o * o;

    throw ArgumentError("Operand's type must be int or Vector");
  }

  bool operator <=(Object o) {
    if (o is Vector)
      return lengthSquared <= o.lengthSquared;
    else if (o is num) return lengthSquared <= o * o;

    throw ArgumentError("Operand's type must be int or Vector");
  }

  Vector operator +(Object o) {
    if (o is Vector)
      return Vector(x + o.x, y + o.y);
    else if (o is int) return Vector(x + o, y + o);

    throw ArgumentError("Operand's type must be int or Vector");
  }

  Vector operator -(Object o) {
    if (o is Vector)
      return Vector(x - o.x, y - o.y);
    else if (o is int) return Vector(x - o, y - o);

    throw ArgumentError("Operand's type must be int or Vector");
  }

  Vector operator *(int scale) => Vector(x * scale, y * scale);

  Vector operator ~/(int scale) => Vector(x ~/ scale, y ~/ scale);

  Vector abs() => Vector(x.abs(), y.abs());

  Vector offset(int x, int y) => Vector(this.x + x, this.y + y);
  Vector offsetX(int offset) => Vector(x + offset, y);
  Vector offsetY(int offset) => Vector(x, y + offset);

  bool contains(Vector o) {
    var right = math.max(0, x);
    if (o.x > right) return false;

    var left = math.min(0, x);
    if (o.x < left) return false;

    var top = math.max(0, y);
    if (o.y > top) return false;

    var bottom = math.min(0, y);
    if (o.y < bottom) return false;

    return true;
  }

  String toString() => "($x, $y)";
}
